@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,8 @@
 #ifndef __INFINOTED_DH_PARAMS_H__
 #define __INFINOTED_DH_PARAMS_H__
 
+#include <infinoted/infinoted-log.h>
+
 #include <libinfinity/common/inf-certificate-credentials.h>
 
 #include <gnutls/gnutls.h>
@@ -28,7 +30,8 @@
 G_BEGIN_DECLS
 
 gboolean
-infinoted_dh_params_ensure(InfCertificateCredentials* creds,
+infinoted_dh_params_ensure(InfinotedLog* log,
+                           InfCertificateCredentials* creds,
                            gnutls_dh_params_t* dh_params,
                            GError** error);
 
