@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,22 +30,11 @@ gboolean
 infinoted_util_create_dirname(const gchar* path,
                               GError** error);
 
-/* TODO: Move this to infinoted-log.[hc] */
-void
-infinoted_util_log_error(const char* fmt, ...);
-
-void
-infinoted_util_log_warning(const char* fmt, ...);
-
-void
-infinoted_util_log_info(const char* fmt, ...);
-
 void
 infinoted_util_set_errno_error(GError** error,
                                int save_errno,
                                const char* prefix);
 
-#ifdef LIBINFINITY_HAVE_LIBDAEMON
 void
 infinoted_util_daemon_set_global_pid_file_proc(void);
 
@@ -54,7 +43,6 @@ infinoted_util_daemon_set_local_pid_file_proc(void);
 
 int
 infinoted_util_daemon_pid_file_kill(int sig);
-#endif
 
 G_END_DECLS
 
