@@ -40,6 +40,8 @@
 #include <string.h>
 #include <errno.h>
 
+#include "config.h"
+
 typedef enum _InfinotedPluginDocumentStreamStatus {
   INFINOTED_PLUGIN_DOCUMENT_STREAM_NORMAL,
   INFINOTED_PLUGIN_DOCUMENT_STREAM_RECEIVING,
@@ -583,7 +585,10 @@ infinoted_plugin_document_stream_subscribe_done(
 
   g_object_get(G_OBJECT(proxy), "session", &session, NULL);
 
-  if(INF_TEXT_IS_SESSION(session))
+  /* User join via document stream only works for chat sessions
+   * at the moment. */
+  if(stream->username == NULL || *stream->username == '\0' ||
+     INF_TEXT_IS_SESSION(session))
   {
     infinoted_plugin_document_stream_start(stream);
   }
@@ -879,7 +884,17 @@ infinoted_plugin_document_stream_send_direct(
 
   do
   {
-    bytes = send(stream->socket, data, len, MSG_NOSIGNAL);
+    bytes = send(
+      stream->socket,
+      data,
+      len,
+#ifdef HAVE_MSG_NOSIGNAL
+      MSG_NOSIGNAL
+#else
+      0
+#endif
+    );
+
     errcode = errno;
 
     if(bytes > 0)
@@ -987,7 +1002,11 @@ infinoted_plugin_document_stream_io_in(
       stream->socket,
       stream->recv_queue.data + queue_offset,
       stream->recv_queue.alloc - queue_offset,
+#ifdef HAVE_MSG_NOSIGNAL
       MSG_NOSIGNAL
+#else
+      0
+#endif
     );
 
     errcode = errno;
