@@ -520,6 +520,11 @@ inf_gtk_browser_store_remove_item(InfGtkBrowserStore* store,
   /* Note we need to reset the browser before we unlink because
    * inf_gtk_browser_store_item_set_browser() requires item still being
    * linked for change notifications. */
+  /* TODO: This is a bit cumbersome. A better way might be to be able for
+   * the iter parameter to be NULL to indicate that an entry with
+   * its browser has been deleted. However, this would still need to do
+   * lots of the unregistration that is now happening in
+   * inf_gtk_browser_store_item_set_browser(). */
   if(item->browser != NULL)
   {
     inf_gtk_browser_store_item_set_browser(store, item, path, NULL);
@@ -1672,6 +1677,7 @@ inf_gtk_browser_store_tree_model_iter_has_child(GtkTreeModel* model,
 {
   InfGtkBrowserStorePrivate* priv;
   InfGtkBrowserStoreItem* item;
+  InfBrowserStatus status;
   InfBrowserIter browser_iter;
 
   priv = INF_GTK_BROWSER_STORE_PRIVATE(model);
@@ -1683,6 +1689,10 @@ inf_gtk_browser_store_tree_model_iter_has_child(GtkTreeModel* model,
   browser_iter.node_id = GPOINTER_TO_UINT(iter->user_data2);
   browser_iter.node = iter->user_data3;
 
+  g_object_get(G_OBJECT(item->browser), "status", &status, NULL);
+  if(status != INF_BROWSER_OPEN)
+    return FALSE;
+
   if(browser_iter.node == NULL)
     inf_browser_get_root(item->browser, &browser_iter);
 
