@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -529,12 +529,15 @@ inf_gtk_io_io_remove_watch(InfIo* io,
                            InfIoWatch* watch)
 {
   InfGtkIoPrivate* priv;
+  guint watch_id;
 
   priv = INF_GTK_IO_PRIVATE(io);
   g_mutex_lock(priv->mutex);
 
   g_assert(g_slist_find(priv->watches, watch) != NULL);
+
   priv->watches = g_slist_remove(priv->watches, watch);
+  watch_id = watch->id;
 
   if(watch->executing)
   {
@@ -555,7 +558,7 @@ inf_gtk_io_io_remove_watch(InfIo* io,
   /* Note that we can do this safely without having locked the mutex because
    * if the callback function is currently being invoked then its user_data
    * will not be destroyed immediately. */
-  g_source_remove(watch->id);
+  g_source_remove(watch_id);
 }
 
 static InfIoTimeout*
