@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -70,7 +70,8 @@ enum {
   PROP_0,
 
   /* construct only */
-  PROP_SESSION
+  PROP_SESSION,
+  PROP_FILENAME
 };
 
 #define INF_ADOPTED_SESSION_RECORD_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_ADOPTED_TYPE_SESSION_RECORD, InfAdoptedSessionRecordPrivate))
@@ -78,6 +79,15 @@ enum {
 static GObjectClass* parent_class;
 static GQuark libxml2_writer_error_quark;
 
+static gint64
+inf_adopted_session_record_get_real_time()
+{
+  /* TODO: Replace by g_get_real_time() once we depend on glib >=2.28 */
+  GTimeVal timeval;
+  g_get_current_time(&timeval);
+  return (gint64)timeval.tv_sec * 1000000 + timeval.tv_usec;
+}
+
 static void
 inf_adopted_session_record_handle_xml_error(InfAdoptedSessionRecord* record)
 {
@@ -152,11 +162,10 @@ inf_adopted_session_record_user_joined(InfAdoptedSessionRecord* record,
 }
 
 static void
-inf_adopted_session_record_execute_request_cb(InfAdoptedAlgorithm* algorithm,
-                                              InfAdoptedUser* user,
-                                              InfAdoptedRequest* request,
-                                              gboolean apply,
-                                              gpointer user_data)
+inf_adopted_session_record_begin_execute_request_cb(InfAdoptedAlgorithm* algo,
+                                                    InfAdoptedUser* user,
+                                                    InfAdoptedRequest* req,
+                                                    gpointer user_data)
 {
   InfAdoptedSessionRecord* record;
   InfAdoptedSessionRecordPrivate* priv;
@@ -173,7 +182,20 @@ inf_adopted_session_record_execute_request_cb(InfAdoptedAlgorithm* algorithm,
   previous = g_hash_table_lookup(priv->last_send_table, user);
   g_assert(previous != NULL);
 
-  session_class->request_to_xml(priv->session, xml, request, previous, FALSE);
+  session_class->request_to_xml(priv->session, xml, req, previous, FALSE);
+
+  inf_xml_util_set_attribute_double(
+    xml,
+    "received",
+    inf_adopted_request_get_receive_time(req) / 1000000.
+  );
+
+  inf_xml_util_set_attribute_double(
+    xml,
+    "executed",
+    inf_adopted_request_get_execute_time(req) / 1000000.
+  );
+
   inf_adopted_session_record_write_node(record, xml);
   xmlFreeNode(xml);
 
@@ -183,8 +205,8 @@ inf_adopted_session_record_execute_request_cb(InfAdoptedAlgorithm* algorithm,
 
   /* Update last send entry */
   previous =
-    inf_adopted_state_vector_copy(inf_adopted_request_get_vector(request));
-  if(inf_adopted_request_affects_buffer(request))
+    inf_adopted_state_vector_copy(inf_adopted_request_get_vector(req));
+  if(inf_adopted_request_affects_buffer(req))
     inf_adopted_state_vector_add(previous, inf_user_get_id(INF_USER(user)), 1);
   g_hash_table_insert(priv->last_send_table, user, previous);
 }
@@ -209,6 +231,13 @@ inf_adopted_session_record_add_user_cb(InfUserTable* user_table,
 
   xml = xmlNewNode(NULL, (const xmlChar*)"user");
   inf_session_user_to_xml(INF_SESSION(priv->session), user, xml);
+
+  inf_xml_util_set_attribute_double(
+    xml,
+    "executed",
+    inf_adopted_session_record_get_real_time() / 1000000.
+  );
+
   inf_adopted_session_record_write_node(record, xml);
   xmlFreeNode(xml);
 
@@ -248,8 +277,8 @@ inf_adopted_session_record_real_start(InfAdoptedSessionRecord* record)
 
   g_signal_connect(
     G_OBJECT(algorithm),
-    "execute-request",
-    G_CALLBACK(inf_adopted_session_record_execute_request_cb),
+    "begin-execute-request",
+    G_CALLBACK(inf_adopted_session_record_begin_execute_request_cb),
     record
   );
 
@@ -410,6 +439,8 @@ inf_adopted_session_record_set_property(GObject* object,
     g_assert(priv->session == NULL); /* construct only */
     priv->session = INF_ADOPTED_SESSION(g_value_dup_object(value));
     break;
+  case PROP_FILENAME:
+    /* read only */
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     break;
@@ -433,6 +464,9 @@ inf_adopted_session_record_get_property(GObject* object,
   case PROP_SESSION:
     g_value_set_object(value, G_OBJECT(priv->session));
     break;
+  case PROP_FILENAME:
+    g_value_set_string(value, priv->filename);
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     break;
@@ -472,6 +506,18 @@ inf_adopted_session_record_class_init(gpointer g_class,
       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY
     )
   );
+
+  g_object_class_install_property(
+    object_class,
+    PROP_FILENAME,
+    g_param_spec_string(
+      "filename",
+      "Filename",
+      "The filename of the record file",
+      NULL,
+      G_PARAM_READABLE
+    )
+  );
 }
 
 GType
@@ -646,6 +692,7 @@ inf_adopted_session_record_start_recording(InfAdoptedSessionRecord* record,
 
   g_assert(priv->filename == NULL);
   priv->filename = g_strdup(filename);
+  g_object_notify(G_OBJECT(record), "filename");
   return TRUE;
 }
 
@@ -703,7 +750,7 @@ inf_adopted_session_record_stop_recording(InfAdoptedSessionRecord* record,
 
       inf_signal_handlers_disconnect_by_func(
         G_OBJECT(algorithm),
-        G_CALLBACK(inf_adopted_session_record_execute_request_cb),
+        G_CALLBACK(inf_adopted_session_record_begin_execute_request_cb),
         record
       );
     }
@@ -750,6 +797,8 @@ inf_adopted_session_record_stop_recording(InfAdoptedSessionRecord* record,
     priv->last_send_table = NULL;
   }
 
+  g_object_notify(G_OBJECT(record), "filename");
+
   return result >= 0;
 }
 
