@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 #include <libinfinity/common/inf-xml-util.h>
 #include <libinfinity/common/inf-error.h>
 #include <libinfinity/inf-i18n.h>
+#include <libinfinity/inf-marshal.h>
 #include <libinfinity/inf-signals.h>
 
 #include <string.h>
@@ -73,6 +74,8 @@ struct _InfAdoptedSessionPrivate {
   InfIoTimeout* noop_timeout;
   /* User to send the time for */
   InfAdoptedSessionLocalUser* next_noop_user;
+  /* Buffer for requests that are not ready to be executed yet */
+  GPtrArray* request_buffer;
 };
 
 enum {
@@ -86,9 +89,17 @@ enum {
   PROP_ALGORITHM
 };
 
+enum {
+  CHECK_REQUEST,
+
+  LAST_SIGNAL
+};
+
 #define INF_ADOPTED_SESSION_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_ADOPTED_TYPE_SESSION, InfAdoptedSessionPrivate))
 
 static InfSessionClass* parent_class;
+static guint session_signals[LAST_SIGNAL];
+
 static GQuark inf_adopted_session_error_quark;
 /* TODO: This should perhaps be a property: */
 static const int INF_ADOPTED_SESSION_NOOP_INTERVAL = 30;
@@ -254,13 +265,19 @@ inf_adopted_session_noop_timeout_func(gpointer user_data)
   g_assert(priv->next_noop_user != NULL);
 
   op = INF_ADOPTED_OPERATION(inf_adopted_no_operation_new());
-  request = inf_adopted_algorithm_generate_request_noexec(
+
+  request = inf_adopted_algorithm_generate_request(
     priv->algorithm,
+    INF_ADOPTED_REQUEST_DO,
     priv->next_noop_user->user,
     op
   );
+
   g_object_unref(op);
 
+  /* There is no need to actually execute the request, since it does not
+   * do anything anyway. */
+
   /* This resets noop_time for this user, determines the next user for
    * which to generate a noop request and schedules the new timeout. */
   inf_adopted_session_broadcast_request(session, request);
@@ -422,6 +439,178 @@ inf_adopted_session_broadcast_n_requests(InfAdoptedSession* session,
   inf_adopted_session_stop_noop_timer(session, local);
 }
 
+static gboolean
+inf_adopted_session_process_request(InfAdoptedSession* session,
+                                    InfAdoptedRequest* request,
+                                    InfAdoptedUser* user,
+                                    GError** error)
+{
+  InfAdoptedSessionPrivate* priv;
+  InfAdoptedStateVector* request_vector;
+  InfAdoptedStateVector* current_vector;
+  gboolean reject_request;
+  GError* local_error;
+  gboolean execute_result;
+
+  xmlNodePtr reply_xml;
+  gchar* request_str;
+  gchar* current_str;
+
+  priv = INF_ADOPTED_SESSION_PRIVATE(session);
+  request_vector = inf_adopted_request_get_vector(request);
+  current_vector = inf_adopted_algorithm_get_current(priv->algorithm);
+
+  if(inf_adopted_state_vector_causally_before(request_vector, current_vector))
+  {
+    g_signal_emit(
+      G_OBJECT(session),
+      session_signals[CHECK_REQUEST],
+      0,
+      request,
+      user,
+      &reject_request
+    );
+
+    local_error = NULL;
+    
+    if(reject_request)
+    {
+      g_set_error(
+        &local_error,
+        inf_adopted_session_error_quark,
+        INF_ADOPTED_SESSION_ERROR_INVALID_REQUEST,
+        "%s",
+        _("The request was rejected via the API")
+      );
+
+      execute_result = FALSE;
+    }
+    else
+    {
+      execute_result = inf_adopted_algorithm_execute_request(
+        priv->algorithm,
+        request,
+        TRUE,
+        &local_error
+      );
+    }
+
+    if(local_error != NULL)
+    {
+      /* Send a message back to where the request came from, to let them
+       * know we couldn't handle this. Note that at the moment this is not
+       * explicitly handled, but it can aid in debugging. */
+      if(inf_user_get_connection(INF_USER(user)) != NULL)
+      {
+        /* Send a message back to where we got this request from, to inform
+         * them that the request cannot be handled. */
+        request_str = inf_adopted_state_vector_to_string(request_vector);
+        current_str = inf_adopted_state_vector_to_string(current_vector);
+
+        reply_xml = xmlNewNode(NULL, (const xmlChar*)"invalid-request");
+
+        inf_xml_util_set_attribute(
+          reply_xml,
+          "request",
+          request_str
+        );
+
+        inf_xml_util_set_attribute(
+          reply_xml,
+          "state",
+          current_str
+        );
+
+        inf_xml_util_set_attribute_uint(
+          reply_xml,
+          "user",
+          inf_user_get_id(INF_USER(user))
+        );
+
+        xmlNewChild(
+          reply_xml,
+          NULL,
+          (const xmlChar*)"reason",
+          (const xmlChar*)local_error->message
+        );
+
+        g_free(request_str);
+        g_free(current_str);
+
+        inf_communication_group_send_message(
+          inf_session_get_subscription_group(INF_SESSION(session)),
+          inf_user_get_connection(INF_USER(user)),
+          reply_xml
+        );
+      }
+
+      g_propagate_error(error, local_error);
+    }
+
+    return execute_result;
+  }
+  else
+  {
+    if(priv->request_buffer == NULL)
+      priv->request_buffer = g_ptr_array_new();
+    g_ptr_array_add(priv->request_buffer, request);
+    g_object_ref(request);
+    return TRUE;
+  }
+}
+
+static void
+inf_adopted_session_process_buffered_requests(InfAdoptedSession* session)
+{
+  InfAdoptedSessionPrivate* priv;
+  InfUserTable* user_table;
+  InfAdoptedStateVector* current;
+
+  guint i;
+  InfAdoptedRequest* request;
+  InfAdoptedStateVector* vector;
+
+  guint user_id;
+  InfUser* user;
+
+  priv = INF_ADOPTED_SESSION_PRIVATE(session);
+
+  if(priv->request_buffer != NULL)
+  {
+    user_table = inf_session_get_user_table(INF_SESSION(session));
+    current = inf_adopted_algorithm_get_current(priv->algorithm);
+    for(i = 0; i < priv->request_buffer->len; ++i)
+    {
+      request =
+        INF_ADOPTED_REQUEST(g_ptr_array_index(priv->request_buffer, i));
+      vector = inf_adopted_request_get_vector(request);
+
+      if(inf_adopted_state_vector_causally_before(vector, current))
+      {
+        g_ptr_array_remove_index_fast(priv->request_buffer, i);
+
+        user_id = inf_adopted_request_get_user_id(request);
+        user = inf_user_table_lookup_user_by_id(user_table, user_id);
+        g_assert(INF_ADOPTED_IS_USER(user));
+
+        /* Note that there is no error handling here, since the buffered
+         * requests are not related to the request which has currently been
+         * received. In order to handle a failure here, the
+         * InfAdoptedAlgorithm::end-execute-request signal should be used. */
+        inf_adopted_session_process_request(
+          session,
+          request,
+          INF_ADOPTED_USER(user),
+          NULL
+        );
+
+        g_object_unref(request);
+        return inf_adopted_session_process_buffered_requests(session);
+      }
+    }
+  }
+}
+
 /*
  * Signal handlers
  */
@@ -517,49 +706,54 @@ inf_adopted_session_constructor_foreach_local_user_func(InfUser* user,
 }
 
 static void
-inf_adopted_session_execute_request_cb(InfAdoptedAlgorithm* algorithm,
-                                       InfAdoptedUser* user,
-                                       InfAdoptedRequest* request,
-                                       gboolean apply,
-                                       gpointer user_data)
+inf_adopted_session_end_execute_request_cb(InfAdoptedAlgorithm* algorithm,
+                                          InfAdoptedUser* user,
+                                          InfAdoptedRequest* request,
+                                          InfAdoptedRequest* translated,
+                                          const GError* error,
+                                          gpointer user_data)
 {
   InfAdoptedSession* session;
   InfAdoptedSessionPrivate* priv;
   GSList* item;
   InfAdoptedSessionLocalUser* local;
   guint id;
+  InfAdoptedOperation* operation;
 
   session = INF_ADOPTED_SESSION(user_data);
   priv = INF_ADOPTED_SESSION_PRIVATE(session);
 
-  if(inf_adopted_request_affects_buffer(request))
+  if(translated != NULL)
   {
-    id = inf_adopted_request_get_user_id(request);
-
-    /* A request has been executed, meaning we are no longer up to date. Send
-     * a noop in some time, so that others know what we already processed. */
-    for(item = priv->local_users; item != NULL; item = g_slist_next(item))
+    if(inf_adopted_request_affects_buffer(translated))
     {
-      local = (InfAdoptedSessionLocalUser*)item->data;
-      if(local->noop_time == 0)
-        /* Except we issued the request ourselves, of course. */
-        if(inf_user_get_id(INF_USER(local->user)) != id)
-          inf_adopted_session_start_noop_timer(session, local);
-    }
-  }
+      id = inf_adopted_request_get_user_id(translated);
 
-  /* Mark inactive users active if they do something */
-  /* Note: This behaviour is implicitly performed by both client and server,
-   * and requires no further network traffic. However, users explictely have
-   * to be set inactive. */
-  if(inf_adopted_request_get_request_type(request) != INF_ADOPTED_REQUEST_DO ||
-     !INF_ADOPTED_IS_NO_OPERATION(inf_adopted_request_get_operation(request)))
-  {
-    /* TODO: We should offer a virtual function to flush all requests for
-     * local users, either here or even in InfSession via a vfunc, so that
-     * we don't accidentally make local users active by a delayed request. */
-    if(inf_user_get_status(INF_USER(user)) == INF_USER_INACTIVE)
-      g_object_set(G_OBJECT(user), "status", INF_USER_ACTIVE, NULL);
+      /* A request has been executed, meaning we are no longer up to date. Send
+       * a noop in some time, so that others know what we already processed. */
+      for(item = priv->local_users; item != NULL; item = g_slist_next(item))
+      {
+        local = (InfAdoptedSessionLocalUser*)item->data;
+        if(local->noop_time == 0)
+          /* Except we issued the request ourselves, of course. */
+          if(inf_user_get_id(INF_USER(local->user)) != id)
+            inf_adopted_session_start_noop_timer(session, local);
+      }
+    }
+
+    /* Mark inactive users active if they do something */
+    /* Note: This behaviour is implicitly performed by both client and server,
+     * and requires no further network traffic. However, users explictely have
+     * to be set inactive. */
+    operation = inf_adopted_request_get_operation(translated);
+    if(!INF_ADOPTED_IS_NO_OPERATION(operation))
+    {
+      /* TODO: We should offer a virtual function to flush all requests for
+       * local users, either here or even in InfSession via a vfunc, so that
+       * we don't accidentally make local users active by a delayed request. */
+      if(inf_user_get_status(INF_USER(user)) == INF_USER_INACTIVE)
+        g_object_set(G_OBJECT(user), "status", INF_USER_ACTIVE, NULL);
+    }
   }
 }
 
@@ -587,8 +781,8 @@ inf_adopted_session_create_algorithm(InfAdoptedSession* session)
 
   g_signal_connect(
     G_OBJECT(priv->algorithm),
-    "execute-request",
-    G_CALLBACK(inf_adopted_session_execute_request_cb),
+    "end-execute-request",
+    G_CALLBACK(inf_adopted_session_end_execute_request_cb),
     session
   );
 
@@ -615,6 +809,7 @@ inf_adopted_session_init(GTypeInstance* instance,
   priv->local_users = NULL;
   priv->noop_timeout = NULL;
   priv->next_noop_user = NULL;
+  priv->request_buffer = NULL;
 }
 
 static GObject*
@@ -692,6 +887,7 @@ inf_adopted_session_dispose(GObject* object)
   InfAdoptedSession* session;
   InfAdoptedSessionPrivate* priv;
   InfUserTable* user_table;
+  guint i;
 
   session = INF_ADOPTED_SESSION(object);
   priv = INF_ADOPTED_SESSION_PRIVATE(session);
@@ -722,11 +918,19 @@ inf_adopted_session_dispose(GObject* object)
 
   g_assert(priv->local_users == NULL);
 
+  if(priv->request_buffer != NULL)
+  {
+    for(i = 0; i < priv->request_buffer->len; ++i)
+      g_object_unref(g_ptr_array_index(priv->request_buffer, i));
+    g_ptr_array_free(priv->request_buffer, TRUE);
+    priv->request_buffer = NULL;
+  }
+
   if(priv->algorithm != NULL)
   {
     inf_signal_handlers_disconnect_by_func(
       G_OBJECT(priv->algorithm),
-      G_CALLBACK(inf_adopted_session_execute_request_cb),
+      G_CALLBACK(inf_adopted_session_end_execute_request_cb),
       session
     );
 
@@ -941,13 +1145,21 @@ inf_adopted_session_process_xml_run(InfSession* session,
   InfAdoptedSessionClass* session_class;
   InfAdoptedRequest* request;
   InfAdoptedUser* user;
+  guint user_id;
+
+  InfAdoptedStateVector* user_vector;
+  InfAdoptedStateVector* request_vector;
 
   gboolean has_num;
+  gboolean process_request;
   guint num;
   GError* local_error;
   InfAdoptedRequest* copy_req;
   guint i;
 
+  gchar* request_str;
+  gchar* user_str;
+
   priv = INF_ADOPTED_SESSION_PRIVATE(session);
 
   if(strcmp((const char*)xml->name, "request") == 0)
@@ -985,11 +1197,17 @@ inf_adopted_session_process_xml_run(InfSession* session,
       g_propagate_error(error, local_error);
       return INF_COMMUNICATION_SCOPE_PTP;
     }
-    
+
+    if(has_num == FALSE)
+      num = 1;
+
+    user_id = inf_user_get_id(INF_USER(user));
+    user_vector = inf_adopted_user_get_vector(user);
+
     request = session_class->xml_to_request(
       INF_ADOPTED_SESSION(session),
       xml,
-      inf_adopted_user_get_vector(user),
+      user_vector,
       FALSE,
       error
     );
@@ -997,33 +1215,113 @@ inf_adopted_session_process_xml_run(InfSession* session,
     if(request == NULL)
       return INF_COMMUNICATION_SCOPE_PTP;
 
-    inf_adopted_algorithm_receive_request(priv->algorithm, request);
+    request_vector = inf_adopted_request_get_vector(request);
 
-    /* Apply the request more than once if num is given. This is mostly used
-     * for multiple undos and redos, but is in general allowed for any
-     * request. */
-    if(has_num)
+    if(!inf_adopted_state_vector_causally_before(user_vector, request_vector))
     {
-      for(i = 1; i < num; ++i)
+      /* Note that this can actually not happen, since the request time is
+       * transferred as a diff to the previous user time. If the absolute
+       * time were transmitted this would need to be handled as an error. */
+      g_assert_not_reached();
+    }
+    else if(inf_adopted_request_get_index(request) !=
+            inf_adopted_state_vector_get(user_vector, user_id))
+    {
+      request_str = inf_adopted_state_vector_to_string(request_vector);
+      user_str = inf_adopted_state_vector_to_string(user_vector);
+
+      g_set_error(
+        error,
+        inf_adopted_session_error_quark,
+        INF_ADOPTED_SESSION_ERROR_INVALID_REQUEST,
+        _("Request \"%s\" by user \"%s\" is not consecutive with respect to "
+          "previously received request \"%s\""),
+        request_str,
+        inf_user_get_name(INF_USER(user)),
+        user_str
+      );
+
+      g_free(request_str);
+      g_free(user_str);
+      g_object_unref(request);
+      return INF_COMMUNICATION_SCOPE_PTP;
+    }
+
+    /* Update the user vector to the state of the request. */
+    user_vector = inf_adopted_state_vector_copy(request_vector);
+    /* Note that this function takes ownership of user_vector */
+    inf_adopted_user_set_vector(INF_ADOPTED_USER(user), user_vector);
+
+    /* Apply the request more than once if num >= 2 is given. This is mostly
+     * used for multiple undos and redos, but is in general allowed for any
+     * request. */
+    for(i = 0; i < num; ++i)
+    {
+      if(i == 0)
+      {
+        copy_req = request;
+        g_object_ref(copy_req);
+      }
+      else
       {
-        /* TODO: This is a bit of a hack since requests are normally
-         * immutable. It avoids an additional vector copy here though. */
         copy_req = inf_adopted_request_copy(request);
 
+        /* TODO: This is a bit of a hack since requests are normally
+         * immutable. It avoids an additional vector copy here though. */
         inf_adopted_state_vector_add(
           inf_adopted_request_get_vector(copy_req),
           inf_user_get_id(INF_USER(user)),
           i
         );
-
-        inf_adopted_algorithm_receive_request(priv->algorithm, copy_req);
-        g_object_unref(copy_req);
       }
+
+      process_request = inf_adopted_session_process_request(
+        INF_ADOPTED_SESSION(session),
+        copy_req,
+        user,
+        error
+      );
+
+      g_object_unref(copy_req);
+
+      /* Update the user vector again, including the component of the processed request. */
+      if(inf_adopted_request_affects_buffer(request))
+      {
+        user_vector = inf_adopted_state_vector_copy(
+          inf_adopted_request_get_vector(copy_req)
+        );
+
+        inf_adopted_state_vector_add(user_vector, user_id, 1);
+        /* Note that this function takes ownership of user_vector */
+        inf_adopted_user_set_vector(INF_ADOPTED_USER(user), user_vector);
+      }
+
+      /* If an error occured then break here, and do not process the
+       * subsequent requests -- they will likely fail as well. */
+      if(process_request == FALSE)
+        break;
     }
 
     g_object_unref(request);
 
-    /* Requests can always be forwarded since user is given. */
+    /* The processed request(s) might have caused some of the buffered
+     * requests to become ready. */
+    if(i > 0)
+    {
+      inf_adopted_session_process_buffered_requests(
+        INF_ADOPTED_SESSION(session)
+      );
+    }
+
+    /* Cleanup requests that are no longer used after
+     * having processed everything */
+    inf_adopted_algorithm_cleanup(
+      inf_adopted_session_get_algorithm(INF_ADOPTED_SESSION(session))
+    );
+
+    /* Requests can always be forwarded since user is given. Explicitly allow
+     * forwarding if the request could not be applied... maybe others are more
+     * lucky? In the worst case it will just fail for them as well. */
     return INF_COMMUNICATION_SCOPE_GROUP;
   }
 
@@ -1099,10 +1397,16 @@ inf_adopted_session_set_xml_user_props(InfSession* session,
                                        guint n_params,
                                        xmlNodePtr xml)
 {
+  InfAdoptedSessionPrivate* priv;
   const GParameter* time;
   InfAdoptedStateVector* vector;
   gchar* time_string;
 
+  const GParameter* id_param;
+  guint id;
+  GSList* item;
+  InfAdoptedSessionLocalUser* local_user;
+
   INF_SESSION_CLASS(parent_class)->set_xml_user_props(
     session,
     params,
@@ -1110,10 +1414,36 @@ inf_adopted_session_set_xml_user_props(InfSession* session,
     xml
   );
 
+  priv = INF_ADOPTED_SESSION_PRIVATE(INF_ADOPTED_SESSION(session));
+
   time = inf_session_lookup_user_property(params, n_params, "vector");
   if(time != NULL)
   {
-    vector = (InfAdoptedStateVector*)g_value_get_boxed(&time->value);
+    /* If this is a local user, use last_send_vector instead of the user's
+     * vector, so that subsequent differential updates are consistent. */
+    vector = NULL;
+    id_param = inf_session_lookup_user_property(params, n_params, "id");
+
+    if(id_param != NULL)
+    {
+      id = g_value_get_uint(&id_param->value);
+      for(item = priv->local_users; item != NULL; item = item->next)
+      {
+        local_user = (InfAdoptedSessionLocalUser*)item->data;
+        if(inf_user_get_id(INF_USER(local_user->user)) == id)
+        {
+          vector = local_user->last_send_vector;
+          break;
+        }
+      }
+    }
+
+    if(vector == NULL)
+    {
+      /* remote user, or a user join request */
+      vector = (InfAdoptedStateVector*)g_value_get_boxed(&time->value);
+    }
+
     time_string = inf_adopted_state_vector_to_string(vector);
     inf_xml_util_set_attribute(xml, "time", time_string);
     g_free(time_string);
@@ -1225,6 +1555,8 @@ inf_adopted_session_synchronization_complete(InfSession* session,
   priv = INF_ADOPTED_SESSION_PRIVATE(session);
   g_object_get(G_OBJECT(session), "status", &status, NULL);
 
+  g_object_freeze_notify(G_OBJECT(session));
+
   INF_SESSION_CLASS(parent_class)->synchronization_complete(
     session,
     connection
@@ -1242,6 +1574,17 @@ inf_adopted_session_synchronization_complete(InfSession* session,
     g_assert(priv->algorithm == NULL);
     inf_adopted_session_create_algorithm(INF_ADOPTED_SESSION(session));
   }
+
+  g_object_thaw_notify(G_OBJECT(session));
+}
+
+static gboolean
+inf_adopted_session_check_request(InfAdoptedSession* session,
+                                  InfAdoptedRequest* request,
+                                  InfAdoptedUser* user)
+{
+  /* Accept all requests by default */
+  return FALSE;
 }
 
 /*
@@ -1284,11 +1627,41 @@ inf_adopted_session_class_init(gpointer g_class,
 
   adopted_session_class->xml_to_request = NULL;
   adopted_session_class->request_to_xml = NULL;
+  adopted_session_class->check_request = inf_adopted_session_check_request;
 
   inf_adopted_session_error_quark = g_quark_from_static_string(
     "INF_ADOPTED_SESSION_ERROR"
   );
 
+  /**
+   * InfAdoptedSession::check-request:
+   * @session: The #InfAdoptedSession which is about to process a request.
+   * @request: The request to be processed.
+   * @user: The user who issued the request.
+   *
+   * This signal is emitted whenever the session received a request from a
+   * non-local user. It is used to decide whether the request should be
+   * processed or not. Note that generally not processing a request results is
+   * loss of synchronization, since other hosts might process the request.
+   * Only if the same condition can be applied on all sites a request should
+   * be rejected. Another possibility is to reject a request at a central host
+   * before it gets distributed to all other clients. If there is one signal
+   * handler returning %TRUE the request is rejected, i.e. only if all signal
+   * handlers return %FALSE it is accepted.
+   */
+  session_signals[CHECK_REQUEST] = g_signal_new(
+    "check-request",
+    G_OBJECT_CLASS_TYPE(object_class),
+    G_SIGNAL_RUN_LAST,
+    G_STRUCT_OFFSET(InfAdoptedSessionClass, check_request),
+    g_signal_accumulator_true_handled, NULL,
+    inf_marshal_BOOLEAN__OBJECT_OBJECT,
+    G_TYPE_BOOLEAN,
+    2,
+    INF_ADOPTED_TYPE_REQUEST,
+    INF_ADOPTED_TYPE_USER
+  );
+
   g_object_class_install_property(
     object_class,
     PROP_IO,
@@ -1432,8 +1805,10 @@ inf_adopted_session_undo(InfAdoptedSession* session,
                          guint n)
 {
   InfAdoptedSessionPrivate* priv;
+  InfAdoptedRequest* first_request;
   InfAdoptedRequest* request;
   guint i;
+  gboolean result;
 
   g_return_if_fail(INF_ADOPTED_IS_SESSION(session));
   g_return_if_fail(INF_ADOPTED_IS_USER(user));
@@ -1442,11 +1817,35 @@ inf_adopted_session_undo(InfAdoptedSession* session,
   /* TODO: Check whether we can issue n undo requests before doing anything */
 
   priv = INF_ADOPTED_SESSION_PRIVATE(session);
-  request = inf_adopted_algorithm_generate_undo(priv->algorithm, user);
-  for(i = 1; i < n; ++i)
-    inf_adopted_algorithm_generate_undo(priv->algorithm, user);
-  inf_adopted_session_broadcast_n_requests(session, request, n);
-  g_object_unref(request);  
+
+  first_request = NULL;
+  for(i = 0; i < n; ++i)
+  {
+    request = inf_adopted_algorithm_generate_request(
+      priv->algorithm,
+      INF_ADOPTED_REQUEST_UNDO,
+      user,
+      NULL
+    );
+
+    result = inf_adopted_algorithm_execute_request(
+      priv->algorithm,
+      request,
+      TRUE,
+      NULL
+    );
+
+    /* This cannot fail if the input parameters have been checked before. */
+    g_assert(result == TRUE);
+
+    if(first_request == NULL)
+      first_request = request;
+    else
+      g_object_unref(request);
+  }
+
+  inf_adopted_session_broadcast_n_requests(session, first_request, n);
+  g_object_unref(first_request);
 }
 
 /**
@@ -1464,8 +1863,10 @@ inf_adopted_session_redo(InfAdoptedSession* session,
                          guint n)
 {
   InfAdoptedSessionPrivate* priv;
+  InfAdoptedRequest* first_request;
   InfAdoptedRequest* request;
   guint i;
+  gboolean result;
 
   /* TODO: Check whether we can issue n redo requests before doing anything */
 
@@ -1474,11 +1875,35 @@ inf_adopted_session_redo(InfAdoptedSession* session,
   g_return_if_fail(n >= 1);
 
   priv = INF_ADOPTED_SESSION_PRIVATE(session);
-  request = inf_adopted_algorithm_generate_redo(priv->algorithm, user);
-  for(i = 1; i < n; ++i)
-    inf_adopted_algorithm_generate_redo(priv->algorithm, user);
-  inf_adopted_session_broadcast_n_requests(session, request, n);
-  g_object_unref(request);
+
+  first_request = NULL;
+  for(i = 0; i < n; ++i)
+  {
+    request = inf_adopted_algorithm_generate_request(
+      priv->algorithm,
+      INF_ADOPTED_REQUEST_REDO,
+      user,
+      NULL
+    );
+
+    result = inf_adopted_algorithm_execute_request(
+      priv->algorithm,
+      request,
+      TRUE,
+      NULL
+    );
+
+    /* This cannot fail if the input parameters have been checked before. */
+    g_assert(result == TRUE);
+
+    if(first_request == NULL)
+      first_request = request;
+    else
+      g_object_unref(request);
+  }
+
+  inf_adopted_session_broadcast_n_requests(session, first_request, n);
+  g_object_unref(first_request);
 }
 
 /**
