@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,19 +25,33 @@
  *
  * #InfAdoptedSplitOperation is a wrapper around that two
  * #InfAdoptedOperation<!-- -->s. This is normally not required directly but
- * may be a result of some transformation.
+ * may be a result of some transformation. It can also be used to atomically
+ * perform multiple operations at once.
+ *
+ * If A denotes the first operation of the split operation and B denotes
+ * the second operation, the split operation applies first A and then B to
+ * the document. Note that a split operation is not commutative, i.e. the
+ * order of the two operations is important and cannot be interchanged at
+ * will. When the second operation, B, is applied, it is assumed that the
+ * operation A was already applied before.
+ *
+ * The reverse of the split operation (A, B) is (R(B), R(A)) where R indicates
+ * the reverse operation. When the split operation is transformed against an
+ * operation T, the result is (T A, (A T) B). When another operation T
+ * is transformed against the split operation, the result is B (A T).
+ * The functions inf_adopted_operation_revert(),
+ * inf_adopted_operation_transform() and
+ * inf_adopted_split_operation_transform_other() perform these three
+ * operations, respectively.
  **/
 
 #include <libinfinity/adopted/inf-adopted-split-operation.h>
 #include <libinfinity/adopted/inf-adopted-operation.h>
 
-#include <libinfinity/adopted/inf-adopted-concurrency-warning.h>
-
 typedef struct _InfAdoptedSplitOperationPrivate InfAdoptedSplitOperationPrivate;
 struct _InfAdoptedSplitOperationPrivate {
   InfAdoptedOperation* first;
   InfAdoptedOperation* second;
-  /* TODO: Cache new_second? We should probably profile before */
 };
 
 enum {
@@ -220,71 +234,120 @@ inf_adopted_split_operation_need_concurrency_id(InfAdoptedOperation* op,
   InfAdoptedSplitOperation* split;
   InfAdoptedSplitOperationPrivate* priv;
 
+  InfAdoptedOperation* new_against;
+  gboolean result;
+
   split = INF_ADOPTED_SPLIT_OPERATION(op);
   priv = INF_ADOPTED_SPLIT_OPERATION_PRIVATE(split);
 
-  return inf_adopted_operation_need_concurrency_id(priv->first, against) ||
-         inf_adopted_operation_need_concurrency_id(priv->second, against);
-}
+  if(inf_adopted_operation_need_concurrency_id(priv->first, against) == TRUE)
+    return TRUE;
 
-static InfAdoptedConcurrencyId
-inf_adopted_split_operation_get_concurrency_id(InfAdoptedOperation* operation,
-                                               InfAdoptedOperation* against)
-{
-  InfAdoptedSplitOperation* split;
-  InfAdoptedSplitOperationPrivate* priv;
-  InfAdoptedConcurrencyId first_id;
-  InfAdoptedConcurrencyId second_id;
+  /* Note that for this transformation there is no concurrency ID required */
+  new_against = inf_adopted_operation_transform(
+    against,
+    priv->first,
+    NULL,
+    NULL,
+    INF_ADOPTED_CONCURRENCY_NONE
+  );
 
-  split = INF_ADOPTED_SPLIT_OPERATION(operation);
-  priv = INF_ADOPTED_SPLIT_OPERATION_PRIVATE(split);
+  result = inf_adopted_operation_need_concurrency_id(
+    priv->second,
+    new_against
+  );
 
-  first_id = inf_adopted_operation_get_concurrency_id(priv->first, against);
-  second_id = inf_adopted_operation_get_concurrency_id(priv->second, against);
-
-  /* everything is fine if both split parts agree, or if only one can
-   * make a decision. Problem if they are contradictory. */
-
-  if(first_id == second_id)
-    return first_id;
-  else if(first_id == INF_ADOPTED_CONCURRENCY_NONE)
-    return second_id;
-  else if(second_id == INF_ADOPTED_CONCURRENCY_NONE)
-    return first_id;
-
-  _inf_adopted_concurrency_warning(INF_ADOPTED_TYPE_SPLIT_OPERATION);
-  return INF_ADOPTED_CONCURRENCY_NONE;
+  g_object_unref(new_against);
+  return result;
 }
 
 static InfAdoptedOperation*
 inf_adopted_split_operation_transform(InfAdoptedOperation* operation,
                                       InfAdoptedOperation* against,
+                                      InfAdoptedOperation* operation_lcs,
+                                      InfAdoptedOperation* against_lcs,
                                       InfAdoptedConcurrencyId concurrency_id)
 {
   InfAdoptedSplitOperation* split;
   InfAdoptedSplitOperationPrivate* priv;
 
   InfAdoptedOperation* new_first;
+  InfAdoptedOperation* new_against;
   InfAdoptedOperation* new_second;
   InfAdoptedOperation* result;
+  
+  InfAdoptedSplitOperationPrivate* priv_lcs;
+  InfAdoptedOperation* first_lcs;
+  InfAdoptedOperation* second_lcs;
+  InfAdoptedOperation* new_against_lcs;
 
   split = INF_ADOPTED_SPLIT_OPERATION(operation);
   priv = INF_ADOPTED_SPLIT_OPERATION_PRIVATE(split);
+  
+  if(INF_ADOPTED_IS_SPLIT_OPERATION(operation_lcs))
+  {
+    g_assert(against_lcs != NULL);
+
+    priv_lcs = INF_ADOPTED_SPLIT_OPERATION_PRIVATE(operation_lcs);
+
+    first_lcs = priv_lcs->first;
+    second_lcs = priv_lcs->second;
+
+    new_against_lcs = inf_adopted_operation_transform(
+      against_lcs,
+      first_lcs,
+      against_lcs,
+      first_lcs,
+      -concurrency_id
+    );
+  }
+  else if(operation_lcs != NULL)
+  {
+    first_lcs = operation_lcs;
+    second_lcs = operation_lcs;
+
+    new_against_lcs = against_lcs;
+    g_object_ref(new_against_lcs);
+  }
+  else
+  {
+    first_lcs = NULL;
+    second_lcs = NULL;
+    new_against_lcs = NULL;
+  }
 
   new_first = inf_adopted_operation_transform(
     priv->first,
     against,
+    first_lcs,
+    against_lcs,
     concurrency_id
   );
 
+  new_against = inf_adopted_operation_transform(
+    against,
+    priv->first,
+    against_lcs,
+    first_lcs,
+    -concurrency_id
+  );
+
   new_second = inf_adopted_operation_transform(
     priv->second,
-    against,
+    new_against,
+    second_lcs,
+    new_against_lcs,
     concurrency_id
   );
 
-  /* TODO: Check whether one of these is a noop and return only the other
-   * one in that case. */
+  if(new_against_lcs != NULL)
+    g_object_unref(new_against_lcs);
+
+  g_object_unref(new_against);
+
+  /* Note that even if one of the two is a no-op, we keep the split operation
+   * at this point. Parts of the split operation implementation relies on the
+   * fact that a split operation is never un-split during transformation. */
 
   result = INF_ADOPTED_OPERATION(
     inf_adopted_split_operation_new(new_first, new_second)
@@ -344,24 +407,96 @@ inf_adopted_split_operation_get_flags(InfAdoptedOperation* operation)
   return result;
 }
 
-static void
+static gboolean
 inf_adopted_split_operation_apply(InfAdoptedOperation* operation,
                                   InfAdoptedUser* by,
-                                  InfBuffer* buffer)
+                                  InfBuffer* buffer,
+                                  GError** error)
 {
   InfAdoptedSplitOperation* split;
   InfAdoptedSplitOperationPrivate* priv;
-  InfAdoptedOperation* new_second;
 
   split = INF_ADOPTED_SPLIT_OPERATION(operation);
   priv = INF_ADOPTED_SPLIT_OPERATION_PRIVATE(split);
 
-  new_second = inf_adopted_operation_transform(priv->second, priv->first, 0);
+  if(!inf_adopted_operation_apply(priv->first, by, buffer, error))
+    return FALSE;
+  if(!inf_adopted_operation_apply(priv->second, by, buffer, error))
+    return FALSE;
+  return TRUE;
+}
 
-  inf_adopted_operation_apply(priv->first, by, buffer);
-  inf_adopted_operation_apply(new_second, by, buffer);
+static InfAdoptedOperation*
+inf_adopted_split_operation_apply_transformed(InfAdoptedOperation* operation,
+                                              InfAdoptedOperation* transformed,
+                                              InfAdoptedUser* by,
+                                              InfBuffer* buffer,
+                                              GError** error)
+{
+  InfAdoptedSplitOperation* split;
+  InfAdoptedSplitOperation* trans_split;
 
-  g_object_unref(G_OBJECT(new_second));
+  InfAdoptedSplitOperationPrivate* priv;
+  InfAdoptedSplitOperationPrivate* trans_priv;
+
+  InfAdoptedOperation* ret_first;
+  InfAdoptedOperation* ret_second;
+  InfAdoptedSplitOperation* result;
+
+  split = INF_ADOPTED_SPLIT_OPERATION(operation);
+  priv = INF_ADOPTED_SPLIT_OPERATION_PRIVATE(split);
+
+  /* The transformed operation must be a split operation, too,
+   * since we do no never unsplit operations when transforming */
+  g_assert(INF_ADOPTED_IS_SPLIT_OPERATION(transformed));
+  trans_split = INF_ADOPTED_SPLIT_OPERATION(transformed);
+  trans_priv = INF_ADOPTED_SPLIT_OPERATION_PRIVATE(trans_split);
+
+  ret_first = inf_adopted_operation_apply_transformed(
+    priv->first,
+    trans_priv->first,
+    by,
+    buffer,
+    error
+  );
+
+  if(ret_first == NULL)
+    return NULL;
+
+  ret_second = inf_adopted_operation_apply_transformed(
+    priv->second,
+    trans_priv->second,
+    by,
+    buffer,
+    error
+  );
+
+  if(ret_second == NULL)
+  {
+    g_object_unref(ret_first);
+    return NULL;
+  }
+
+  if(ret_first == priv->first && ret_second == priv->second)
+  {
+    /* No operation was modifide to be reversible; skip this case */
+    g_object_unref(ret_first);
+    g_object_unref(ret_second);
+    return operation;
+  }
+  else
+  {
+    /* Otherwise create a new operation */
+    result = inf_adopted_split_operation_new(
+      ret_first,
+      ret_second
+    );
+
+    g_object_unref(ret_first);
+    g_object_unref(ret_second);
+
+    return INF_ADOPTED_OPERATION(result);
+  }
 }
 
 static InfAdoptedOperation*
@@ -369,7 +504,6 @@ inf_adopted_split_operation_revert(InfAdoptedOperation* operation)
 {
   InfAdoptedSplitOperation* split;
   InfAdoptedSplitOperationPrivate* priv;
-  InfAdoptedOperation* new_second;
 
   InfAdoptedOperation* revert_first;
   InfAdoptedOperation* revert_second;
@@ -378,13 +512,10 @@ inf_adopted_split_operation_revert(InfAdoptedOperation* operation)
   split = INF_ADOPTED_SPLIT_OPERATION(operation);
   priv = INF_ADOPTED_SPLIT_OPERATION_PRIVATE(split);
 
-  new_second = inf_adopted_operation_transform(priv->second, priv->first, 0);
-
   revert_first = inf_adopted_operation_revert(priv->first);
-  revert_second = inf_adopted_operation_revert(new_second);
-  g_object_unref(new_second);
+  revert_second = inf_adopted_operation_revert(priv->second);
 
-  result = inf_adopted_split_operation_new(revert_first, revert_second);
+  result = inf_adopted_split_operation_new(revert_second, revert_first);
 
   g_object_unref(revert_first);
   g_object_unref(revert_second);
@@ -392,34 +523,6 @@ inf_adopted_split_operation_revert(InfAdoptedOperation* operation)
   return INF_ADOPTED_OPERATION(result);
 }
 
-static InfAdoptedOperation*
-inf_adopted_split_operation_make_reversible(InfAdoptedOperation* operation,
-                                            InfAdoptedOperation* with,
-                                            InfBuffer* buffer)
-{
-  InfAdoptedSplitOperation* split;
-  InfAdoptedSplitOperationPrivate* priv;
-  InfAdoptedOperation* first;
-  InfAdoptedOperation* second;
-
-  split = INF_ADOPTED_SPLIT_OPERATION(operation);
-  priv = INF_ADOPTED_SPLIT_OPERATION_PRIVATE(split);
-
-  first = inf_adopted_operation_make_reversible(priv->first, with, buffer);
-  if(first == NULL) return NULL;
-
-  second = inf_adopted_operation_make_reversible(priv->second, with, buffer);
-  if(second == NULL)
-  {
-    g_object_unref(G_OBJECT(first));
-    return NULL;
-  }
-
-  return INF_ADOPTED_OPERATION(
-    inf_adopted_split_operation_new(first, second)
-  );
-}
-
 static void
 inf_adopted_split_operation_operation_init(gpointer g_iface,
                                            gpointer iface_data)
@@ -429,13 +532,12 @@ inf_adopted_split_operation_operation_init(gpointer g_iface,
 
   iface->need_concurrency_id =
     inf_adopted_split_operation_need_concurrency_id;
-  iface->get_concurrency_id = inf_adopted_split_operation_get_concurrency_id;
   iface->transform = inf_adopted_split_operation_transform;
   iface->copy = inf_adopted_split_operation_copy;
   iface->get_flags = inf_adopted_split_operation_get_flags;
   iface->apply = inf_adopted_split_operation_apply;
+  iface->apply_transformed = inf_adopted_split_operation_apply_transformed;
   iface->revert = inf_adopted_split_operation_revert;
-  iface->make_reversible = inf_adopted_split_operation_make_reversible;
 }
 
 GType
@@ -535,6 +637,8 @@ inf_adopted_split_operation_unsplit(InfAdoptedSplitOperation* operation)
  * inf_adopted_split_operation_transform_other:
  * @op: A #InfAdoptedSplitOperation.
  * @other: An arbitrary #InfAdoptedOperation.
+ * @op_lcs: The operation @op at a previous state, or %NULL.
+ * @other_lcs: The operation @other at a previous state, or %NULL.
  * @concurrency_id: The concurrency id for the transformation of
  * @other against @op.
  *
@@ -545,24 +649,77 @@ inf_adopted_split_operation_unsplit(InfAdoptedSplitOperation* operation)
 InfAdoptedOperation*
 inf_adopted_split_operation_transform_other(InfAdoptedSplitOperation* op,
                                             InfAdoptedOperation* other,
+                                            InfAdoptedOperation* op_lcs,
+                                            InfAdoptedOperation* other_lcs,
                                             gint concurrency_id)
 {
   InfAdoptedSplitOperationPrivate* priv;
+  InfAdoptedSplitOperationPrivate* priv_lcs;
   InfAdoptedOperation* tmp;
-  InfAdoptedOperation* new_second;
   InfAdoptedOperation* result;
 
+  InfAdoptedOperation* first_lcs;
+  InfAdoptedOperation* second_lcs;
+  InfAdoptedOperation* tmp_lcs;
+
   g_return_val_if_fail(INF_ADOPTED_IS_SPLIT_OPERATION(op), NULL);
   g_return_val_if_fail(INF_ADOPTED_IS_OPERATION(other), NULL);
 
   priv = INF_ADOPTED_SPLIT_OPERATION_PRIVATE(op);
+  
+  if(INF_ADOPTED_IS_SPLIT_OPERATION(op_lcs))
+  {
+    g_assert(other_lcs != NULL);
 
-  tmp = inf_adopted_operation_transform(other, priv->first, concurrency_id);
-  new_second = inf_adopted_operation_transform(priv->second, priv->first, 0);
-  result = inf_adopted_operation_transform(tmp, new_second, concurrency_id);
+    priv_lcs = INF_ADOPTED_SPLIT_OPERATION_PRIVATE(op_lcs);
+    first_lcs = priv_lcs->first;
+    second_lcs = priv_lcs->second;
 
-  g_object_unref(G_OBJECT(tmp));
-  g_object_unref(G_OBJECT(new_second));
+    tmp_lcs = inf_adopted_operation_transform(
+      other_lcs,
+      first_lcs,
+      other_lcs,
+      first_lcs,
+      concurrency_id
+    );
+  }
+  else if(op_lcs != NULL)
+  {
+    g_assert(other_lcs != NULL);
+
+    first_lcs = op_lcs;
+    second_lcs = op_lcs;
+
+    tmp_lcs = other_lcs;
+    g_object_ref(tmp_lcs);
+  }
+  else
+  {
+    first_lcs = NULL;
+    second_lcs = NULL;
+    tmp_lcs = NULL;
+  }
+
+  tmp = inf_adopted_operation_transform(
+    other,
+    priv->first,
+    other_lcs,
+    first_lcs,
+    concurrency_id
+  );
+
+  result = inf_adopted_operation_transform(
+    tmp,
+    priv->second,
+    tmp_lcs,
+    second_lcs,
+    concurrency_id
+  );
+
+  if(tmp_lcs != NULL)
+    g_object_unref(tmp_lcs);
+
+  g_object_unref(tmp);
   return result;
 }
 
