@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -63,7 +63,7 @@ struct _InfAdoptedStateVector {
 };
 
 static gsize
-inf_adopted_state_vector_find_insert_pos(InfAdoptedStateVector* vec,
+inf_adopted_state_vector_find_insert_pos(const InfAdoptedStateVector* vec,
                                          guint id)
 {
   gsize begin;
@@ -99,8 +99,8 @@ inf_adopted_state_vector_find_insert_pos(InfAdoptedStateVector* vec,
 }
 
 static InfAdoptedStateVectorComponent*
-inf_adopted_state_vector_lookup(InfAdoptedStateVector* vec,
-                                      guint id)
+inf_adopted_state_vector_lookup(const InfAdoptedStateVector* vec,
+                                guint id)
 {
   gsize pos;
 
@@ -255,7 +255,7 @@ inf_adopted_state_vector_free(InfAdoptedStateVector* vec)
  * Return Value: The @component'th entry in the vector.
  */
 guint
-inf_adopted_state_vector_get(InfAdoptedStateVector* vec,
+inf_adopted_state_vector_get(const InfAdoptedStateVector* vec,
                              guint id)
 {
   InfAdoptedStateVectorComponent* comp;
@@ -339,7 +339,7 @@ inf_adopted_state_vector_add(InfAdoptedStateVector* vec,
  * which @func will not be called if their timestamp is 0.
  **/
 void
-inf_adopted_state_vector_foreach(InfAdoptedStateVector* vec,
+inf_adopted_state_vector_foreach(const InfAdoptedStateVector* vec,
                                  InfAdoptedStateVectorForeachFunc func,
                                  gpointer user_data)
 {
@@ -366,8 +366,8 @@ inf_adopted_state_vector_foreach(InfAdoptedStateVector* vec,
  * Return Value: -1, 0 or 1.
  **/
 int
-inf_adopted_state_vector_compare(InfAdoptedStateVector* first,
-                                 InfAdoptedStateVector* second)
+inf_adopted_state_vector_compare(const InfAdoptedStateVector* first,
+                                 const InfAdoptedStateVector* second)
 {
   gsize first_pos;
   gsize second_pos;
@@ -461,8 +461,8 @@ inf_adopted_state_vector_compare(InfAdoptedStateVector* first,
  * Return Value: Whether @second depends on @first.
  **/
 gboolean
-inf_adopted_state_vector_causally_before(InfAdoptedStateVector* first,
-                                         InfAdoptedStateVector* second)
+inf_adopted_state_vector_causally_before(const InfAdoptedStateVector* first,
+                                         const InfAdoptedStateVector* second)
 {
   gsize first_pos;
   gsize second_pos;
@@ -537,9 +537,10 @@ inf_adopted_state_vector_causally_before(InfAdoptedStateVector* first,
  * @inc_component<!-- -->th component increased by one.
  **/
 gboolean
-inf_adopted_state_vector_causally_before_inc(InfAdoptedStateVector* first,
-                                             InfAdoptedStateVector* second,
-                                             guint inc_component)
+inf_adopted_state_vector_causally_before_inc(
+  const InfAdoptedStateVector* first,
+  const InfAdoptedStateVector* second,
+  guint inc_component)
 {
   gsize first_pos;
   gsize second_pos;
@@ -648,8 +649,8 @@ inf_adopted_state_vector_causally_before_inc(InfAdoptedStateVector* first,
  * @second.
  */
 guint
-inf_adopted_state_vector_vdiff(InfAdoptedStateVector* first,
-                               InfAdoptedStateVector* second)
+inf_adopted_state_vector_vdiff(const InfAdoptedStateVector* first,
+                               const InfAdoptedStateVector* second)
 {
   gsize n;
   guint first_sum;
@@ -681,7 +682,7 @@ inf_adopted_state_vector_vdiff(InfAdoptedStateVector* first,
  * Return Value: A newly-allocated string to be freed by the caller.
  **/
 gchar*
-inf_adopted_state_vector_to_string(InfAdoptedStateVector* vec)
+inf_adopted_state_vector_to_string(const InfAdoptedStateVector* vec)
 {
   GString* str;
   gsize pos;
@@ -804,8 +805,8 @@ inf_adopted_state_vector_from_string(const gchar* str,
  * Return Value: A newly allocated string to be freed by the caller.
  **/
 gchar*
-inf_adopted_state_vector_to_string_diff(InfAdoptedStateVector* vec,
-                                        InfAdoptedStateVector* orig)
+inf_adopted_state_vector_to_string_diff(const InfAdoptedStateVector* vec,
+                                        const InfAdoptedStateVector* orig)
 {
   gsize vec_pos;
   gsize orig_pos;
@@ -821,6 +822,8 @@ inf_adopted_state_vector_to_string_diff(InfAdoptedStateVector* vec,
     NULL
   );
 
+  g_assert(vec->data != NULL || vec->size == 0);
+
   str = g_string_sized_new(vec->size * 12);
   vec_pos = 0;
 
@@ -829,7 +832,7 @@ inf_adopted_state_vector_to_string_diff(InfAdoptedStateVector* vec,
     orig_comp = orig->data + orig_pos;
     if(orig_comp->n == 0) continue;
 
-    g_assert(vec_pos != vec->size);
+    g_assert(vec_pos < vec->size);
     vec_comp = vec->data + vec_pos;
     while(vec_comp->id < orig_comp->id)
     {
@@ -840,7 +843,7 @@ inf_adopted_state_vector_to_string_diff(InfAdoptedStateVector* vec,
 
       ++vec_pos;
 
-      g_assert(vec_pos != vec->size);
+      g_assert(vec_pos < vec->size);
       vec_comp = vec->data + vec_pos;
     }
 
@@ -865,7 +868,7 @@ inf_adopted_state_vector_to_string_diff(InfAdoptedStateVector* vec,
 
   /* All remaining components in vec have no counterpart in orig, meaning
    * their values in orig are implicitely zero. */
-  while(vec_pos != vec->size)
+  while(vec_pos < vec->size)
   {
     vec_comp = vec->data + vec_pos;
     if (vec_comp->n > 0)
@@ -895,7 +898,7 @@ inf_adopted_state_vector_to_string_diff(InfAdoptedStateVector* vec,
  **/
 InfAdoptedStateVector*
 inf_adopted_state_vector_from_string_diff(const gchar* str,
-                                          InfAdoptedStateVector* orig,
+                                          const InfAdoptedStateVector* orig,
                                           GError** error)
 {
   InfAdoptedStateVector* vec;
