@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -193,6 +193,22 @@ inf_chat_buffer_reserve_message(InfChatBuffer* buffer,
         );
       }
     }
+    else
+    {
+      /* No memmoving has to be performed */
+      g_assert(n == 0 || n == priv->num_messages);
+
+      if(n == 0)
+      {
+        g_assert(priv->first_message > 0);
+        --priv->first_message;
+      }
+      else
+      {
+        g_assert(priv->first_message + priv->num_messages <
+                 priv->alloc_messages);
+      }
+    }
 
     ++ priv->num_messages;
   }
