@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -91,7 +91,7 @@ inf_chat_session_get_type(void) G_GNUC_CONST;
 
 InfChatSession*
 inf_chat_session_new(InfCommunicationManager* manager,
-                     guint backlog_size,
+                     InfChatBuffer* buffer,
                      InfSessionStatus status,
                      InfCommunicationGroup* sync_group,
                      InfXmlConnection* sync_connection);
