@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -483,19 +483,24 @@ inf_discovery_avahi_service_browser_callback(AvahiServiceBrowser* browser,
   switch(event)
   {
   case AVAHI_BROWSER_NEW:
-    discovery_info = g_slice_new(InfDiscoveryInfo);
-    discovery_info->service_name = g_strdup(name);
-    discovery_info->service_type = info->type;
-    discovery_info->domain = g_strdup(domain);
-    discovery_info->interface = interface;
-    discovery_info->protocol = protocol;
+    /* Ignore what we published ourselves */
+    if((flags & AVAHI_LOOKUP_RESULT_OUR_OWN) == 0)
+    {
+      discovery_info = g_slice_new(InfDiscoveryInfo);
+      discovery_info->service_name = g_strdup(name);
+      discovery_info->service_type = info->type;
+      discovery_info->domain = g_strdup(domain);
+      discovery_info->interface = interface;
+      discovery_info->protocol = protocol;
 
-    discovery_info->service_resolver = NULL;
-    discovery_info->resolved = NULL;
-    discovery_info->resolv = NULL;
+      discovery_info->service_resolver = NULL;
+      discovery_info->resolved = NULL;
+      discovery_info->resolv = NULL;
+
+      info->discovered = g_slist_prepend(info->discovered, discovery_info);
+      inf_discovery_discovered(INF_DISCOVERY(avahi), discovery_info);
+    }
 
-    info->discovered = g_slist_prepend(info->discovered, discovery_info);
-    inf_discovery_discovered(INF_DISCOVERY(avahi), discovery_info);
     break;
   case AVAHI_BROWSER_REMOVE:
     for(item = info->discovered; item != NULL; item = g_slist_next(item))
@@ -543,19 +548,39 @@ inf_discovery_avahi_service_browser_callback(AvahiServiceBrowser* browser,
 static void
 inf_discovery_avahi_entry_group_add_service(InfLocalPublisherItem* item)
 {
-  /* TODO: Error handling */
-  avahi_entry_group_add_service(
-    item->entry_group,
-    AVAHI_IF_UNSPEC,
-    AVAHI_PROTO_UNSPEC,
-    0,
-    item->name,
-    item->type,
-    NULL,
-    NULL,
-    item->port,
-    NULL
-  );
+  char* new_name;
+  int res;
+
+  do
+  {
+    res = avahi_entry_group_add_service(
+      item->entry_group,
+      AVAHI_IF_UNSPEC,
+      AVAHI_PROTO_UNSPEC,
+      0,
+      item->name,
+      item->type,
+      NULL,
+      NULL,
+      item->port,
+      NULL
+    );
+
+    if(res != AVAHI_ERR_COLLISION) break;
+
+    new_name = avahi_alternative_service_name(item->name);
+    avahi_free(item->name);
+    item->name = new_name;
+  } while(1);
+
+  if(res != 0)
+  {
+    g_warning(
+      "Failed to publish service \"%s\" via avahi: %s",
+      item->name,
+      avahi_strerror(res)
+    );
+  }
 }
 
 static void
@@ -585,7 +610,6 @@ inf_discovery_avahi_entry_group_callback(AvahiEntryGroup* group,
     avahi_free(item->name);
     item->name = new_name;
 
-    /* TODO: Error handling */
     avahi_entry_group_reset(item->entry_group);
     inf_discovery_avahi_entry_group_add_service(item);
     avahi_entry_group_commit(item->entry_group);
