@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -71,7 +71,7 @@ struct _InfSaslContextSession {
   Gsasl_session* session;
   gpointer session_data;
   InfIo* main_io;
-  InfStandaloneIo* session_io;
+  GAsyncQueue* session_queue;
   /* query or stepped dispatch, protected by context mutex */
   /* TODO: This is required because Inf(StandaloneIo) can not guarantee to
    * return before the dispatch is executed in another thread. We would not
@@ -79,6 +79,10 @@ struct _InfSaslContextSession {
    * InfIoDispatch pointer before executing the dispatch. */
   InfIoDispatch* dispatch;
   GThread* thread;
+  /* This flag tells whether we are currently processing user data in the
+   * helper thread. It is meant as a simple indicator in the main thread
+   * whether more data can be given to the context or not. */
+  gboolean stepping;
 
   /* used in session thread only */
   gchar* step64;
@@ -255,13 +259,11 @@ inf_sasl_context_session_message_func(gpointer user_data)
   case INF_SASL_CONTEXT_MESSAGE_TERMINATE:
     /* session thread */
     message->session->status = INF_SASL_CONTEXT_SESSION_TERMINATE;
-    inf_standalone_io_loop_quit(message->session->session_io);
     break;
   case INF_SASL_CONTEXT_MESSAGE_CONTINUE:
     /* session thread */
     g_assert(message->session->status == INF_SASL_CONTEXT_SESSION_INNER);
     message->session->retval = message->shared.cont.retval;
-    inf_standalone_io_loop_quit(message->session->session_io);
     break;
   case INF_SASL_CONTEXT_MESSAGE_STEP:
     /* session thread */
@@ -274,7 +276,6 @@ inf_sasl_context_session_message_func(gpointer user_data)
     message->shared.step.data = NULL; /* prevent deletion */
 
     message->session->status = INF_SASL_CONTEXT_SESSION_INNER;
-    inf_standalone_io_loop_quit(message->session->session_io);
     break;
   case INF_SASL_CONTEXT_MESSAGE_QUERY:
     /* main thread */
@@ -317,6 +318,9 @@ inf_sasl_context_session_message_func(gpointer user_data)
       break;
     }
 
+    g_assert(message->session->stepping == TRUE);
+    message->session->stepping = FALSE;
+
     message->shared.stepped.func(
       message->session,
       message->shared.stepped.data,
@@ -340,6 +344,8 @@ inf_sasl_context_gsasl_callback(Gsasl* gsasl,
                                 Gsasl_property prop)
 {
   InfSaslContextSession* session;
+  InfSaslContextMessage* message;
+
   session = (InfSaslContextSession*)gsasl_session_hook_get(gsasl_session);
 
   /* if the status is TERMINATE then get out of the inner loop by
@@ -360,8 +366,14 @@ inf_sasl_context_gsasl_callback(Gsasl* gsasl,
 
   g_mutex_unlock(session->context->mutex);
 
-  /* wait for continue */
-  inf_standalone_io_loop(session->session_io);
+  session->retval = G_MAXINT;
+  while(session->status == INF_SASL_CONTEXT_SESSION_INNER &&
+        session->retval == G_MAXINT)
+  {
+    message = g_async_queue_pop(session->session_queue);
+    inf_sasl_context_session_message_func(message);
+    inf_sasl_context_message_free(message);
+  }
 
   g_mutex_lock(session->context->mutex);
 
@@ -377,6 +389,8 @@ static void*
 inf_sasl_context_thread_func(gpointer data)
 {
   InfSaslContextSession* session;
+  InfSaslContextMessage* message;
+
   session = (InfSaslContextSession*)data;
 
   int retval;
@@ -390,11 +404,15 @@ inf_sasl_context_thread_func(gpointer data)
     switch(session->status)
     {
     case INF_SASL_CONTEXT_SESSION_OUTER:
-      inf_standalone_io_loop(session->session_io);
+      message = g_async_queue_pop(session->session_queue);
+      inf_sasl_context_session_message_func(message);
+      inf_sasl_context_message_free(message);
       break;
     case INF_SASL_CONTEXT_SESSION_INNER:
       g_mutex_lock(session->context->mutex);
 
+      g_assert(session->dispatch == NULL);
+
       /* This might call the gsasl callback once or more in which we wait
        * for input from the main thread. */
       retval = gsasl_step64(
@@ -446,7 +464,6 @@ inf_sasl_context_thread_func(gpointer data)
         if(output) gsasl_free(output);
       }
 
-
       break;
     case INF_SASL_CONTEXT_SESSION_TERMINATE:
     default:
@@ -477,9 +494,12 @@ inf_sasl_context_start_session(InfSaslContext* context,
   session->session_data = session_data;
   session->main_io = io;
   g_object_ref(session->main_io);
-  session->session_io = inf_standalone_io_new();
+  session->session_queue =
+    g_async_queue_new_full(inf_sasl_context_message_free);
   session->dispatch = NULL;
   session->thread = NULL;
+  session->stepping = FALSE;
+
   session->status = INF_SASL_CONTEXT_SESSION_OUTER;
   session->step64 = NULL;
   session->retval = GSASL_OK;
@@ -499,7 +519,7 @@ inf_sasl_context_start_session(InfSaslContext* context,
   if(session->thread == NULL)
   {
     context->sessions = g_slist_remove(context->sessions, session);
-    g_object_unref(session->session_io);
+    g_async_queue_unref(session->session_queue);
     g_object_unref(session->main_io);
     g_slice_free(InfSaslContextSession, session);
     return NULL;
@@ -952,11 +972,9 @@ inf_sasl_context_stop_session(InfSaslContext* context,
   g_mutex_unlock(context->mutex);
 
   /* Tell client thread to terminate */
-  inf_io_add_dispatch(
-    INF_IO(session->session_io),
-    inf_sasl_context_session_message_func,
-    inf_sasl_context_message_terminate(session),
-    inf_sasl_context_message_free
+  g_async_queue_push(
+    session->session_queue,
+    inf_sasl_context_message_terminate(session)
   );
 
   g_thread_join(session->thread);
@@ -969,7 +987,11 @@ inf_sasl_context_stop_session(InfSaslContext* context,
   gsasl_finish(session->session);
   g_mutex_unlock(context->mutex);
 
-  g_object_unref(session->session_io);
+  /* Note that this assertion should hold because us pushing the terminate
+   * message into the end of the queue, and all other queued messages will
+   * have been processed before. */
+  g_async_queue_unref(session->session_queue);
+
   g_object_unref(session->main_io);
 
   g_free(session->step64);
@@ -1044,11 +1066,9 @@ inf_sasl_context_session_continue(InfSaslContextSession* session,
 {
   g_return_if_fail(session != NULL);
 
-  inf_io_add_dispatch(
-    INF_IO(session->session_io),
-    inf_sasl_context_session_message_func,
-    inf_sasl_context_message_continue(session, retval),
-    inf_sasl_context_message_free
+  g_async_queue_push(
+    session->session_queue,
+    inf_sasl_context_message_continue(session, retval)
   );
 }
 
@@ -1075,14 +1095,32 @@ inf_sasl_context_session_feed(InfSaslContextSession* session,
 {
   g_return_if_fail(session != NULL);
   g_return_if_fail(func != NULL);
+  g_return_if_fail(session->stepping == FALSE);
   /*g_return_if_fail(session->context->callback != NULL); not threadsafe */
 
-  inf_io_add_dispatch(
-    INF_IO(session->session_io),
-    inf_sasl_context_session_message_func,
-    inf_sasl_context_message_step(session, data, func, user_data),
-    inf_sasl_context_message_free
+  session->stepping = TRUE;
+
+  g_async_queue_push(
+    session->session_queue,
+    inf_sasl_context_message_step(session, data, func, user_data)
   );
 }
 
+/**
+ * inf_sasl_context_session_is_processing:
+ * @session: A #InfSaslContextSession.
+ *
+ * Returns whether the session is currently asynchronously processing data
+ * fed to it with inf_sasl_context_session_feed(). In this case the first
+ * call needs to finish before another one is allowed to be made.
+ *
+ * Returns: Whether @session is currently processing data asynchronously.
+ */
+gboolean
+inf_sasl_context_session_is_processing(InfSaslContextSession* session)
+{
+  g_return_val_if_fail(session != NULL, FALSE);
+  return session->stepping;
+}
+
 /* vim:set et sw=2 ts=2: */
