@@ -35,6 +35,7 @@
 
 #include <libinftext/inf-text-fixline-buffer.h>
 #include <libinftext/inf-text-user.h>
+#include <libinftext/inf-text-move-operation.h>
 #include <libinfinity/common/inf-buffer.h>
 #include <libinfinity/inf-signals.h>
 
@@ -577,6 +578,9 @@ inf_text_fixline_buffer_text_inserted_cb(InfTextBuffer* buffer,
   guint buffer_length;
   guint end;
 
+  guint user_pos;
+  gint user_len;
+
   InfTextChunk* new_chunk;
   gchar stext[16];
   gchar* text;
@@ -633,11 +637,35 @@ inf_text_fixline_buffer_text_inserted_cb(InfTextBuffer* buffer,
       g_free(priv->keep);
       priv->keep = NULL;
     }
+
+    /* TODO: We don't know whether this was an insert-caret or not, but
+     * assume for now that it was. Advance the user's caret, which would
+     * account for the added newlines that we swallowed above. */
+    /* TODO: A better way might be to just skip this special handling
+     * altogether and just actually propagate this change. */
+    user_pos = inf_text_user_get_caret_position(INF_TEXT_USER(user));
+    user_len = inf_text_user_get_selection_length(INF_TEXT_USER(user));
+
+    inf_text_move_operation_transform_insert(
+      pos,
+      chunk_length,
+      &user_pos,
+      &user_len,
+      FALSE
+    );
+
+    inf_text_user_set_selection(
+      INF_TEXT_USER(user),
+      user_pos,
+      user_len,
+      TRUE
+    );
   }
   else
   {
     if(pos > end)
     {
+      /* TODO: Should we handle caret updates here as well? */
       g_assert(priv->n_keep < 0);
       g_assert((guint)(-priv->n_keep) >= pos - end);
 
@@ -701,6 +729,9 @@ inf_text_fixline_buffer_text_erased_cb(InfTextBuffer* buffer,
   guint use_keep;
   InfTextChunk* new_chunk;
 
+  guint user_pos;
+  gint user_len;
+
   priv = INF_TEXT_FIXLINE_BUFFER_PRIVATE(user_data);
 
   chunk_length = inf_text_chunk_get_length(chunk);
@@ -759,6 +790,28 @@ inf_text_fixline_buffer_text_erased_cb(InfTextBuffer* buffer,
         use_keep
       );
     }
+
+    /* TODO: We don't know whether this was an erase-caret or not, but
+     * assume for now that it was. Advance the user's caret, which would
+     * account for the removed newlines that we swallowed above. */
+    /* TODO: A better way might be to just skip this special handling
+     * altogether and just actually propagate this change. */
+    user_pos = inf_text_user_get_caret_position(INF_TEXT_USER(user));
+    user_len = inf_text_user_get_selection_length(INF_TEXT_USER(user));
+
+    inf_text_move_operation_transform_delete(
+      pos,
+      chunk_length,
+      &user_pos,
+      &user_len
+    );
+
+    inf_text_user_set_selection(
+      INF_TEXT_USER(user),
+      user_pos,
+      user_len,
+      TRUE
+    );
   }
   else
   {
@@ -773,6 +826,7 @@ inf_text_fixline_buffer_text_erased_cb(InfTextBuffer* buffer,
     else if(pos + chunk_length > end)
     {
       /* Propagate partly */
+      /* TODO: Should we handle caret updates here as well? */
       g_assert(priv->n_keep < 0);
       g_assert(chunk_length - (end - pos) <= (guint)(-priv->n_keep));
 
@@ -1203,6 +1257,9 @@ inf_text_fixline_buffer_buffer_insert_text(InfTextBuffer* buffer,
     fixline_buffer
   );
 
+  /* Notify */
+  inf_text_buffer_text_inserted(buffer, pos, chunk, user);
+
   /* Keep the number of lines at the end fixed */
   inf_text_fixline_buffer_fix_lines(fixline_buffer);
 }
@@ -1215,6 +1272,7 @@ inf_text_fixline_buffer_buffer_erase_text(InfTextBuffer* buffer,
 {
   InfTextFixlineBuffer* fixline_buffer;
   InfTextFixlineBufferPrivate* priv;
+  InfTextChunk* erased_content;
   guint buf_len;
 
   fixline_buffer = INF_TEXT_FIXLINE_BUFFER(buffer);
@@ -1227,6 +1285,8 @@ inf_text_fixline_buffer_buffer_erase_text(InfTextBuffer* buffer,
     fixline_buffer
   );
 
+  erased_content = inf_text_buffer_get_slice(buffer, pos, len);
+
   if(pos + len > buf_len)
   {
     if(pos > buf_len)
@@ -1287,6 +1347,10 @@ inf_text_fixline_buffer_buffer_erase_text(InfTextBuffer* buffer,
     fixline_buffer
   );
 
+  /* Notify */
+  inf_text_buffer_text_erased(buffer, pos, erased_content, user);
+  inf_text_chunk_free(erased_content);
+
   /* Keep the number of lines at the end fixed */
   inf_text_fixline_buffer_fix_lines(fixline_buffer);
 }
@@ -1374,7 +1438,7 @@ inf_text_fixline_buffer_buffer_create_end_iter(InfTextBuffer* buffer)
     }
 
     iter = g_slice_new(InfTextBufferIter);
-    iter->base_iter = NULL;
+    iter->base_iter = base_iter;
   }
 
   return iter;
@@ -1474,7 +1538,8 @@ inf_text_fixline_buffer_buffer_iter_prev(InfTextBuffer* buffer,
     if(iter->keep_begin == 0)
     {
       iter->base_iter = inf_text_buffer_create_end_iter(priv->buffer);
-      g_assert(iter->base_iter != NULL);
+      if(iter->base_iter == NULL)
+        return FALSE;
 
       return TRUE;
     }
