@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,6 +25,7 @@
 
 #include <libinfinity/adopted/inf-adopted-split-operation.h>
 #include <libinfinity/adopted/inf-adopted-operation.h>
+#include <libinfinity/inf-i18n.h>
 
 /* recon is helper information to reconstruct the original delete operation.
  * It stores the parts that have been erased from the remote delete operation
@@ -291,22 +292,11 @@ inf_text_remote_delete_operation_need_concurrency_id(
   );
 }
 
-static InfAdoptedConcurrencyId
-inf_text_remote_delete_operation_get_concurrency_id(
-  InfAdoptedOperation* operation,
-  InfAdoptedOperation* against)
-{
-  g_assert(INF_TEXT_IS_REMOTE_DELETE_OPERATION(operation));
-
-  return inf_text_delete_operation_get_concurrency_id(
-    INF_TEXT_DELETE_OPERATION(operation),
-    against
-  );
-}
-
 static InfAdoptedOperation*
 inf_text_remote_delete_operation_transform(InfAdoptedOperation* operation,
                                            InfAdoptedOperation* against,
+                                           InfAdoptedOperation* operation_lcs,
+                                           InfAdoptedOperation* against_lcs,
                                            InfAdoptedConcurrencyId cid)
 {
   g_assert(INF_TEXT_IS_REMOTE_DELETE_OPERATION(operation));
@@ -315,16 +305,14 @@ inf_text_remote_delete_operation_transform(InfAdoptedOperation* operation,
   {
     return inf_text_delete_operation_transform_insert(
       INF_TEXT_DELETE_OPERATION(operation),
-      INF_TEXT_INSERT_OPERATION(against),
-      cid
+      INF_TEXT_INSERT_OPERATION(against)
     );
   }
   else if(INF_TEXT_IS_DELETE_OPERATION(against))
   {
     return inf_text_delete_operation_transform_delete(
       INF_TEXT_DELETE_OPERATION(operation),
-      INF_TEXT_DELETE_OPERATION(against),
-      cid
+      INF_TEXT_DELETE_OPERATION(against)
     );
   }
   else
@@ -368,10 +356,11 @@ inf_text_remote_delete_operation_get_flags(InfAdoptedOperation* operation)
   return INF_ADOPTED_OPERATION_AFFECTS_BUFFER;
 }
 
-static void
+static gboolean
 inf_text_remote_delete_operation_apply(InfAdoptedOperation* operation,
                                        InfAdoptedUser* by,
-                                       InfBuffer* buffer)
+                                       InfBuffer* buffer,
+                                       GError** error)
 {
   InfTextRemoteDeleteOperationPrivate* priv;
 
@@ -380,24 +369,45 @@ inf_text_remote_delete_operation_apply(InfAdoptedOperation* operation,
 
   priv = INF_TEXT_REMOTE_DELETE_OPERATION_PRIVATE(operation);
 
-  inf_text_buffer_erase_text(
-    INF_TEXT_BUFFER(buffer),
-    priv->position,
-    priv->length,
-    INF_USER(by)
-  );
+  if(priv->position + priv->length >
+     inf_text_buffer_get_length(INF_TEXT_BUFFER(buffer)))
+  {
+    g_set_error(
+      error,
+      g_quark_from_static_string("INF_TEXT_OPERATION_ERROR"),
+      INF_TEXT_OPERATION_ERROR_INVALID_DELETE,
+      "%s",
+      _("Attempt to remove text from after the end of the document")
+    );
+
+    return FALSE;
+  }
+  else
+  {
+    inf_text_buffer_erase_text(
+      INF_TEXT_BUFFER(buffer),
+      priv->position,
+      priv->length,
+      INF_USER(by)
+    );
+    
+    return TRUE;
+  }
 }
 
 static InfAdoptedOperation*
-inf_text_remote_delete_operation_make_reversible(InfAdoptedOperation* op,
-                                                 InfAdoptedOperation* with,
-                                                 InfBuffer* buffer)
+inf_text_remote_delete_operation_apply_transformed(InfAdoptedOperation* op,
+                                                   InfAdoptedOperation* trans,
+                                                   InfAdoptedUser* by,
+                                                   InfBuffer* buffer,
+                                                   GError** error)
 {
   InfTextRemoteDeleteOperationPrivate* priv;
   InfTextChunk* chunk;
   InfTextChunk* temp_slice;
   GSList* list;
   GSList* item;
+  InfAdoptedOperation* operation;
   GSList* recon_list;
   GSList* recon_item;
   InfTextRemoteDeleteOperationRecon* recon;
@@ -409,15 +419,15 @@ inf_text_remote_delete_operation_make_reversible(InfAdoptedOperation* op,
   /* TODO: We can probably optimize this function, but then we should
    * a) profile it and b) in many cases input parameters to this function
    * are trivial anyway. */
-  if(INF_ADOPTED_IS_SPLIT_OPERATION(with))
+  if(INF_ADOPTED_IS_SPLIT_OPERATION(trans))
   {
     list = inf_adopted_split_operation_unsplit(
-      INF_ADOPTED_SPLIT_OPERATION(with)
+      INF_ADOPTED_SPLIT_OPERATION(trans)
     );
   }
   else
   {
-    list = g_slist_prepend(NULL, with);
+    list = g_slist_prepend(NULL, trans);
   }
 
   chunk = inf_text_chunk_new(
@@ -430,6 +440,8 @@ inf_text_remote_delete_operation_make_reversible(InfAdoptedOperation* op,
     g_assert(INF_TEXT_IS_REMOTE_DELETE_OPERATION(item->data));
     priv = INF_TEXT_REMOTE_DELETE_OPERATION_PRIVATE(item->data);
 
+    operation = INF_ADOPTED_OPERATION(item->data);
+
     if(priv->length > 0)
     {
       temp_slice = inf_text_buffer_get_slice(
@@ -469,13 +481,19 @@ inf_text_remote_delete_operation_make_reversible(InfAdoptedOperation* op,
     /* Free recon list if newly allocated */
     if(priv->length > 0)
       inf_text_remote_delete_operation_recon_free(recon_list);
+
+    if(!inf_adopted_operation_apply(operation, by, buffer, error))
+    {
+      g_slist_free(list);
+      inf_text_chunk_free(chunk);
+      return NULL;
+    }
   }
 
   g_slist_free(list);
 
   priv = INF_TEXT_REMOTE_DELETE_OPERATION_PRIVATE(op);
   result = inf_text_default_delete_operation_new(priv->position, chunk);
-
   inf_text_chunk_free(chunk);
 
   return INF_ADOPTED_OPERATION(result);
@@ -533,7 +551,6 @@ inf_text_remote_delete_operation_transform_overlap(
   GObject* result;
   InfTextRemoteDeleteOperationPrivate* result_priv;
 
-
   /* It is actually possible that two remote delete operations are
    * transformed against each other (actually the parts of a splitted
    * remote delete operation). However, they must not overlap. */
@@ -627,7 +644,7 @@ inf_text_remote_delete_operation_transform_split(
   
   second_operation = g_object_new(
     INF_TEXT_TYPE_REMOTE_DELETE_OPERATION,
-    "position", priv->position + split_pos + split_len,
+    "position", priv->position + split_len,
     "length", priv->length - split_pos,
     NULL
   );
@@ -702,15 +719,14 @@ inf_text_remote_delete_operation_operation_init(gpointer g_iface,
 
   iface->need_concurrency_id =
     inf_text_remote_delete_operation_need_concurrency_id;
-  iface->get_concurrency_id =
-    inf_text_remote_delete_operation_get_concurrency_id;
   iface->transform = inf_text_remote_delete_operation_transform;
   iface->copy = inf_text_remote_delete_operation_copy;
   iface->get_flags = inf_text_remote_delete_operation_get_flags;
   iface->apply = inf_text_remote_delete_operation_apply;
+  iface->apply_transformed =
+    inf_text_remote_delete_operation_apply_transformed;
   /* RemoteDeleteOperation is not reversible */
   iface->revert = NULL;
-  iface->make_reversible = inf_text_remote_delete_operation_make_reversible;
 }
 
 static void
@@ -793,11 +809,11 @@ inf_text_remote_delete_operation_get_type(void)
  * characters starting from position @position. Note that this operation is
  * not reversible because it does not know the text to delete and is therefore
  * only used to transmit a delete operation through the network to reduce
- * bandwidth usage. The other part can then reconstruct the deleted text
- * using the make_reversible vfunc.
+ * bandwidth usage. The remote part can then reconstruct the deleted text
+ * using inf_adopted_operation_apply_transformed().
  *
  * However, it is easier to just use #InfTextDefaultDeleteOperation if you
- * want the  operation to be reversible.
+ * want the operation to be reversible.
  *
  * Return Value: A new #InfTextRemoteDeleteOperation.
  **/
